@@ -114,6 +114,22 @@ int main ( int argc, char **argv )
             {
               (void) SetLogEventMask(argv[++arg]);
             }
+          else if (LocaleCompare("define",option+1) == 0)
+            {
+              if (arg == argc)
+                {
+                  (void) printf("-define argument missing\n");
+                  (void) fflush(stdout);
+                  exit_status = 1;
+                  goto program_exit;
+                }
+              if (AddDefinitions(imageInfo,argv[++arg],&exception)
+                  == MagickFail)
+                {
+                  exit_status = 1;
+                  goto program_exit;
+                }
+            }
           else if (LocaleCompare("depth",option+1) == 0)
             {
               imageInfo->depth=QuantumDepth;
@@ -184,9 +200,10 @@ int main ( int argc, char **argv )
   if (arg != argc-2)
     {
       (void) printf("arg=%d, argc=%d\n", arg, argc);
-      (void) printf ( "Usage: %s [-compress algorithm -debug events -depth "
-		      "integer -filespec spec -log format -nocheck "
-		      " -quality quality -size geometry -stdio -verbose]"
+      (void) printf ( "Usage: %s [-compress algorithm] [-debug events]"
+                      " [-depth integer] [-define value] [-filespec spec]"
+                      " [-log format] [-nocheck] [-quality quality]"
+                      " [-size geometry] -stdio -verbose]"
                       " infile format\n", argv[0] );
       (void) fflush(stdout);
       exit_status = 1;
@@ -244,6 +261,8 @@ int main ( int argc, char **argv )
     }
   if (exception.severity != UndefinedException)
     {
+      if (exception.reason)
+        (void) printf("    reason:%s\n",exception.reason);
       CatchException(&exception);
       exit_status = 1;
       goto program_exit;
@@ -315,6 +334,8 @@ int main ( int argc, char **argv )
   original->delay = 10;
   if (!WriteImage ( imageInfo, original ))
     {
+      if (exception.reason)
+        (void) printf("    reason:%s\n",exception.reason);
       CatchException(&original->exception);
       exit_status = 1;
       goto program_exit;
@@ -359,6 +380,8 @@ int main ( int argc, char **argv )
       }
     if (exception.severity != UndefinedException)
       {
+        if (exception.reason)
+          (void) printf("    reason:%s\n",exception.reason);
 	CatchException(&exception);
 	(void) fflush(stderr);
 	ping_error = MagickTrue;
@@ -407,6 +430,8 @@ int main ( int argc, char **argv )
     }
   if (exception.severity != UndefinedException)
     {
+      if (exception.reason)
+        (void) printf("    reason:%s\n",exception.reason);
       CatchException(&exception);
       exit_status = 1;
       goto program_exit;
@@ -445,6 +470,8 @@ int main ( int argc, char **argv )
   (void) fflush(stdout);
   if(!WriteImage (imageInfo,original))
     {
+      if (exception.reason)
+        (void) printf("    reason:%s\n",exception.reason);
       CatchException(&original->exception);
       exit_status = 1;
       goto program_exit;
@@ -476,6 +503,8 @@ int main ( int argc, char **argv )
   final = ReadImage ( imageInfo, &exception );
   if (exception.severity != UndefinedException)
     {
+      if (exception.reason)
+        (void) printf("    reason:%s\n",exception.reason);
       CatchException(&exception);
       exit_status = 1;
       goto program_exit;
@@ -512,6 +541,7 @@ int main ( int argc, char **argv )
           (!strcmp( "PCD", format )) ||
           (!strcmp( "PCDS", format )) ||
           (!strcmp( "UYVY", format )) ||
+          (!strcmp( "WEBP", format )) ||
           (!strcmp( "YUV", format )) ||
           (final->compression == JPEGCompression))
         fuzz_factor = 0.06;
